"""Directional parity encoder.

Python implementation of the directional parity algorithum used in
Radio Lab's Cicada project (http://project.wnyc.org/cicadas)
"""
from errors import OddBitLengthsOnly


def to_int(seq):
    lookup = {True: '1', False: '0'}
    seq = list(seq)
    seq =  map(bool, map(int, seq))
    seq = map(lookup.get, seq)
    return int(''.join(seq), 2)

def decode(seq):
    """Decode a directional parity encodede value.
    
    Args: 
      seq: A sequence of "boolean looking" objects such as [0, 1, 0], [False, True, False] or "010"
    Returns:
      An integer representing its value
    """

    OddBitLengthsOnly.raise_if_qualifies(len(seq))
    seq = map(bool, map(int, seq))
    bits = len(seq)

    low =  to_int(list(reversed(seq[:bits//2])))
    center = seq[bits//2]
    high = to_int(seq[bits//2 + 1:])

    if center and low == high:
        return 2**(bits-1) + low
    
    if  center ^ (low > high):
        return decode(list(reversed(seq)))

    return low | high << (bits // 2)
    
    
def main(argv):
    if len(argv) <= 1:
        print "Usage: %s bit0 bit1 bit2 ... bitn # (n is odd)" % (argv[0],)
        print
        print "See http://github.com/wnyc/directional_parity for more details"
    print decode(map(int, argv[1:]))
    return 0





from decoder import decode

def main(argv):
    if len(argv) <= 1:
        print "Usage: %s bit0 bit1 bit2 ... bitn # (n is odd)" % (argv[0],)
        print
        print "See http://github.com/wnyc/directional_parity for more details"
    value = float(decode(map(int, argv[1:])))
    f = (value * 9.0 - 80.0) / 20.0
    c = (value / 4.0) - 20.0
    print "%0.2fC %0.2fF" % (c, f)
    return 0





"""Directional parity encoder.

Python implementation of the directional parity algorithum used in
Radio Lab's Cicada project (http://project.wnyc.org/cicadas)
"""
from errors import OutOfRange

def bitify(i, bits):
    """Convert a number into a base 2 series of bits.  LSB at the end."""
    try:
        v = map(bool, map(int, bin(i).replace('0b', '')))
        if len(v) < bits:
            v[:0] = [0] * (bits - len(v))
        return v
    except ValueError:
        raise ValueError("Bad %d" % i)

def encode_extended_range(bits, value):
    value -= 2 ** (bits-1)
    nibble =  bitify(value, bits//2)
    bits = []
    bits.extend(reversed(nibble))
    bits.append(True)
    bits.extend(nibble)
    return bits

def encode(bit_length, value):
    """Encode a value as a direction partiy encoded sequence of bits."""
    OutOfRange.raise_if_qualifies(bit_length, value)

    if value >= 2**(bit_length - 1):
        return encode_extended_range(bit_length, value)

    mask = 2**(bit_length/2) - 1
    lower = value & mask
    upper = (value >> bit_length//2) & mask
    center = lower > upper
    bits = bitify(lower, bit_length//2)
    bits.reverse()
    bits.append(center)
    bits.extend(bitify(upper, bit_length//2))
    map(bool, bits)
    return bits


def limit(bits):
    return OutOfRange.limit(bits)

def main(argv):
    if len(argv) not in (2,3):
        print "Usage: %s <# of bits> (<value>)\n"
        print 
        print "If value is provided, returns the encoding, otherwise returns the largest value that could be encoded."
        print
        print "See http://github.com/wnyc/directional_parity for more details"
        return 1
    argv = map(int, argv[1:])
    if len(argv) == 1:
        print limit(*argv)
    else:
        print ' '.join(map({True:'1', False: '0'}.get, encode(*argv)))
    return 0




class OddBitLengthsOnly(ValueError):
    """Wrong number of bits
    
    Directional parity encoded values must have an odd number of bits.
    """

    @property 
    def bits(self):
        "The number of bits the user attempted to use."
        return self.__bits

    def __init__(self, bits):
        self.__bits = bits
        ValueError.__init__(self, "Directional Parity bit lengths must be an odd positive integer, not %d" % (self.bits))

    @classmethod
    def raise_if_qualifies(cls, bits):
        if bits % 2 == 0:
            raise cls(bits)


class OutOfRange(ValueError):
    """OutOfRange
    
    Attempted to encode a negative number or too large a positive number.  The directional parity encoding sceheme can only encode a value within the range [0, 2**(b-1) + 2**(b/2)] where b is the number of bits.
    """
                                
    def __init__(self, bits, value):
        self.__bits = bits
        self.__value = value
        if value < 0:
            problem = "%d is negative." % value 
        else:
            problem = "%d is too large." % value
        ValueError.__init__(self, "Can only encode positive numbers not greater than %d in an %d bit directional paritiy number.  %s" % (self.limit(bits), bits, problem))

    @staticmethod
    def limit(bits):
        if bits < 0:
            raise ValueError("The number of bits passed to %s.limit must be odd and greater than 2.  %d does not quality" % (self.__class__.__name__, bits))
        return 2**(bits-1) + 2**(bits//2) - 1

    @classmethod
    def raise_if_qualifies(cls, bits, value):
        OddBitLengthsOnly.raise_if_qualifies(bits)
        if value > cls.limit(bits):
            raise cls(bits, value)
        



#!/usr/bin/env python
"""
directional_parity
======


"""

from setuptools import setup


setup(
    name='directional_parity',
    version='0.0.0',
    author='Adam DePrince',
    author_email='adeprince@nypublicradio.org',
    description='Direction detecting parity - developed for project.wnyc.org/cicadas',
    long_description=__doc__,
    py_modules = [
        'directional_parity/__init__',
        'directional_parity/encoder',
        'directional_parity/decoder',
        'directional_parity/temperature',
        ],
    packages = ["directional_parity"],
    zip_safe=True,
    include_package_data=True,
    classifiers=[
        ],
    scripts = [
        'scripts/directional_parity_encode',
        'scripts/directional_parity_decode',
        'scripts/wnyc_cicada_decode_temp',
        ],
    url = "https://github.com/adamdeprince/wnyc_systems",
    install_requires = [
        ]
)

